//	Player.cpp - The player's ship.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Player.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "Sounds.h"

#include <cmath>

//------------------------------------------------------------------------------
uint32_t Player::image = 0;
uint32_t Player::particle = 0;
uint32_t Player::debris[3] = {0, 0, 0};

//------------------------------------------------------------------------------
Player::Player():
xPos(-128.0f),
yPos(370.5f),
xSpeed(3.0f),
ySpeed(0.0f),
dead(false),
bossTime(false),
upIsDown(false),
leftIsDown(false),
rightIsDown(false),
downIsDown(false)
{
	particles.setDensity(0.5f);
	particles.setDirection(-0.5f, 0.0f);
	particles.setLifespan(10.0f);
	particles.setColour(1.0f, 1.0f, 1.0f, 0.5f);
	particles.setSize(2.0f, 2.0f);

	explosion.setLifespan(30.0f);
	explosion.setDensity(4.0f);
	explosion.setStartSize(getWidth(), getHeight());
	explosion.setColour(0.75f, 0.75f, 0.75f, 0.75f);
	explosion.setDirection(-1.0f, -0.33f);

	if(particle)
	{
		particles.addImage(particle);
		particles.start();

		explosion.addImage(particle);
		explosion.addImage(debris[0]);
		explosion.addImage(debris[1]);
		explosion.addImage(debris[2]);
	}
}

//------------------------------------------------------------------------------
Player::~Player()
{
	
}

//------------------------------------------------------------------------------
void Player::move()
{
	float x = xPos;
	float y = yPos;
	const float inertia = 0.925f;

	if(xPos < 0.0f)
	{
		xPos += xSpeed;

		if(upIsDown && !downIsDown)
		{
			ySpeed = -3.0f;
			yPos += ySpeed;
		}
		else if(downIsDown && !upIsDown)
		{
			ySpeed = 3.0f;
			yPos += ySpeed;
		}
		else if(fabsf(ySpeed) > 0.0f)
		{
			ySpeed *= inertia;
			yPos += ySpeed;

			if(fabsf(ySpeed) < 0.01f)
				ySpeed = 0.0f;
		}
	}
	else
	{
		if(upIsDown && !downIsDown)
		{
			ySpeed = -3.0f;
			yPos += ySpeed;
		}
		else if(downIsDown && !upIsDown)
		{
			ySpeed = 3.0f;
			yPos += ySpeed;
		}
		else if(fabsf(ySpeed) > 0.0f)
		{
			ySpeed *= inertia;
			yPos += ySpeed;

			if(fabsf(ySpeed) < 0.01f)
				ySpeed = 0.0f;
		}

		if(leftIsDown && !rightIsDown)
		{
			xSpeed = -3.0f;
			xPos += xSpeed;
		}
		else if(rightIsDown && !leftIsDown)
		{
			xSpeed = 3.0f;
			xPos += xSpeed;
		}
		else if(fabsf(xSpeed) > 0.0f)
		{
			xSpeed *= inertia;
			xPos += xSpeed;

			if(fabsf(xSpeed) < 0.01f)
				xSpeed = 0.0f;
		}

		if(xPos < 0.0f)
			xPos = x;
		else if(xPos > (1280.0f-120.0f))
			xPos = x;
		if(yPos < 44.0f)
			kill();
		else if(yPos > (746.0f-59.0f))
			kill();

		if(bossTime)
		{
			if(xPos > (1280.0f-512.0f-getWidth()))
				xPos -= 3.0f;
		}
	}
}

//------------------------------------------------------------------------------
void Player::draw(Drawer *d)
{
	if(!image)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::player_png, Images::player_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::playersmoke_png, Images::playersmoke_pngSize);
		particle = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris1_png, Images::debris1_pngSize);
		debris[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris2_png, Images::debris2_pngSize);
		debris[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::debris3_png, Images::debris3_pngSize);
		debris[2] = d->loadImage(data.data, data.width, data.height);

		particles.addImage(particle);
		particles.start();

		explosion.addImage(particle);
		explosion.addImage(debris[0]);
		explosion.addImage(debris[1]);
		explosion.addImage(debris[2]);
	}

	if(!dead)
	{
		particles.setPosition(xPos + 15.0f, yPos + 52.0f);
		particles.move();
		particles.draw(d);
		d->drawImage(image, xPos, yPos);
	}
	else
	{
		explosion.move();
		explosion.draw(d);
	}

	if(0)
		d->fillRect(getX(), getY(), getWidth(), getHeight(), 255, 0, 0, 128);
}

//------------------------------------------------------------------------------
void Player::kill()
{
	dead = true;
	explosion.setPosition(getX(), getY());
	explosion.start(15);
	Sounds::getInstance().playSound(Sounds::PlayerDie);
}

//------------------------------------------------------------------------------
void Player::setBossTime()
{
	bossTime = true;
}

//------------------------------------------------------------------------------
void Player::moveUp(bool val)
{
	upIsDown = val;
}

//------------------------------------------------------------------------------
void Player::moveLeft(bool val)
{
	leftIsDown = val;
}

//------------------------------------------------------------------------------
void Player::moveRight(bool val)
{
	rightIsDown = val;
}

//------------------------------------------------------------------------------
void Player::moveDown(bool val)
{
	downIsDown = val;
}
